import { Metadata, Root, Snapshot, TargetFile, Targets, Timestamp } from '@tufjs/models';
import { KeyPair } from './key';
export declare function createTargetsMeta(targetFiles: TargetFile[], keyPair: KeyPair): Metadata<Targets>;
export declare function createSnapshotMeta(targets: Metadata<Targets>, keyPair: KeyPair): Metadata<Snapshot>;
export declare function createTimestampMeta(snapshot: Metadata<Snapshot>, keyPair: KeyPair): Metadata<Timestamp>;
export declare function createRootMeta(keyPair: KeyPair): Metadata<Root>;
