import { TUFRepo } from './repo';
import { Handler } from './shared.types';
export interface TUFHandlerOptions {
    metadataPathPrefix?: string;
    targetPathPrefix?: string;
}
export declare function tufHandlers(tufRepo: TUFRepo, opts: TUFHandlerOptions): Handler[];
