import { Metadata, Root, Snapshot, Targets, Timestamp } from '@tufjs/models';
import type { Target } from './shared.types';
export interface TUFRepo {
    rootMeta: Metadata<Root>;
    timestampMeta: Metadata<Timestamp>;
    snapshotMeta: Metadata<Snapshot>;
    targetsMeta: Metadata<Targets>;
    targets: Target[];
}
export declare function initializeTUFRepo(targets: Target[]): TUFRepo;
