import crypto from 'crypto';
export type { KeyObject } from 'crypto';
export declare function generateKeyPair(): crypto.KeyPairKeyObjectResult;
export declare function digestSHA256(data: crypto.BinaryLike): string;
export declare function signSHA256(key: crypto.KeyObject, data: Buffer): string;
